import os
import subprocess
import json
import requests
from datetime import datetime, timedelta

def load_config():
    try:
        with open('config.json', 'r') as config_file:
            config = json.load(config_file)
            return config
    except FileNotFoundError:
        print("Brak pliku konfiguracyjnego 'config.json'.")
        exit(1)
    except json.JSONDecodeError:
        print("Błąd w parsowaniu pliku konfiguracyjnego 'config.json'. Sprawdź poprawność formatu JSON.")
        exit(1)

def backup_files(config):
    backup_folder = config['backup_folder']
    timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
    backup_archive = f"{backup_folder}/backup_{timestamp}.tar.gz"

    # Tworzenie kopii zapasowej plików z kilku ścieżek źródłowych
    source_paths = config['source_folders']

    # Oczekiwanie na zakończenie poprzedniego polecenia
    subprocess.run(['tar', 'czf', backup_archive, '-C', '/'] + source_paths, check=True)

    return backup_archive

def backup_database(config):
    timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
    database_backup_file = f"{config['backup_folder']}/db_backup_{timestamp}.sql"

    # Tworzenie ogólnej kopii zapasowej wszystkich baz danych
    subprocess.run([
        'mysqldump',
        '-u', config['db_username'],
        '-p' + config['db_password'],
        '-h', config['db_host'],
        '--all-databases'
    ], stdout=open(database_backup_file, 'w'))

    return database_backup_file

def cleanup_old_backups(config):
    backup_folder = config['backup_folder']
    cleanup_threshold = datetime.now() - timedelta(days=config['cleanup_days'])

    # Usuwanie starszych kopii zapasowych
    for filename in os.listdir(backup_folder):
        if filename.startswith('db_backup_'):
            # Sprawdź, czy nazwa pliku ma wystarczającą długość
            if len(filename) >= 21:
                file_date_str = ''.join(c for c in filename if c.isdigit())
                if file_date_str:
                    try:
                        file_date = datetime.strptime(file_date_str, '%Y%m%d%H%M%S')
                        if file_date < cleanup_threshold:
                            file_path = os.path.join(backup_folder, filename)
                            os.remove(file_path)
                            print(f"Usunięto starszą kopię zapasową: {filename}")
                    except ValueError:
                        print(f"Nieprawidłowy format daty w nazwie pliku: {filename}")
                else:
                    print(f"Brak cyfr w nazwie pliku: {filename}")
            else:
                print(f"Nieprawidłowy format nazwy pliku: {filename}")

def send_discord_notification(config, success=True):
    webhook_url = config.get('discord_webhook', '')

    if webhook_url:
        status = "Powodzenie" if success else "Niepowodzenie"
        message = f"{status} - Wysyłanie kopii zapasowej na serwer: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
        
        payload = {
            "content": message
        }

        try:
            response = requests.post(webhook_url, json=payload)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            print(f"Błąd podczas wysyłania powiadomienia na Discord: {e}")

def rsync_to_remote(config):
    subprocess.run([
        'rsync', '-avz',
        config['backup_folder'] + '/',
        f"{config['remote_user']}@{config['remote_host']}:{config['remote_backup_folder']}"
    ], check=True)

def main():
    config = load_config()

    # Wykonaj kopię zapasową plików i bazy danych
    backup_archive = backup_files(config)
    database_backup_file = backup_database(config)

    # Sprawdź i usuń starsze kopie zapasowe
    cleanup_old_backups(config)

    # Przesyłanie kopii zapasowych na serwer docelowy
    try:
        rsync_to_remote(config)
        send_discord_notification(config, success=True)
    except subprocess.CalledProcessError:
        print("Błąd podczas wysyłania kopii zapasowej na serwer docelowy.")
        send_discord_notification(config, success=False)

if __name__ == "__main__":
    main()